@@ -37,7 +37,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.31.1.3 Thu, 16 Nov 2000 11:40:09 +1100 kaos $"
+#ident "$Id: depmod.c 1.35 Sat, 09 Dec 2000 23:59:14 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -159,6 +159,8 @@ typedef struct MODULE {
 		SYMBOL **symtab;
 		int n_syms;
 	} defsym, undefs;
+	char **generic_string;
+	int n_generic_string;
 	struct pci_device_id *pci_device;
 	int n_pci_device;
 	struct isapnp_device_id *isapnp_device;
@@ -354,6 +356,34 @@ static const char has_kernel_changed[] =
 	"\n\tyou probably need a new version of modutils to handle this kernel."
 	"\n\tCheck linux/Documentation/Changes.";
 
+/*
+ *	Extract any generic string from the given symbol.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_generic_string(struct obj_file *f, MODULE *mod)
+{
+	struct obj_section *sec;
+	char *p, *ep, *s, *v;
+	
+	sec = obj_find_section(f, ".modstring");
+	if (sec == NULL)
+		return;
+	p = sec->contents;
+	ep = p + sec->header.sh_size;
+	while (p < ep) {
+		s = p;
+		while (*p != '\0' && p < ep)
+			p++;
+		if (*p == '\0')
+			p++;
+		if (s - p > 1)		/* empty string? */
+			continue;
+		mod->generic_string = xrealloc(mod->generic_string, ++(mod->n_generic_string)*sizeof(char *));
+		v = mod->generic_string[mod->n_generic_string-1] = xmalloc(p-s);
+		strcpy(v, s);
+	}
+}
+
 /*
  *	Extract any pci_device_table from the module.
  *	Note: assumes same machine and arch for depmod and module.
@@ -658,6 +688,7 @@ static int loadobj(const char *objname, int ignore_suffix)
 		error("depmod obj_relocate failed\n");
 		return(-1);
 	}
+	extract_generic_string(f, mod);
 	image = xmalloc(m_size);
 	obj_create_image(f, image);
 	extract_pci_device_id(f, image, m_size, mod);
@@ -850,6 +881,7 @@ char *shortname(const char *name)
 static void prtdepend(char *base_dir, int nflag)
 {
 	FILE *dep = stdout;
+	FILE *generic_string = stdout;
 	FILE *pcimap = stdout;
 	FILE *isapnpmap = stdout;
 	FILE *usbmap = stdout;
@@ -860,6 +892,7 @@ static void prtdepend(char *base_dir, int nflag)
 
 	if (!nflag) {
 		dep = gen_file_open(gen_file+GEN_DEPFILE);
+		generic_string = gen_file_open(gen_file+GEN_GENERIC_STRINGFILE);
 		pcimap = gen_file_open(gen_file+GEN_PCIMAPFILE);
 		isapnpmap = gen_file_open(gen_file+GEN_ISAPNPMAPFILE);
 		usbmap = gen_file_open(gen_file+GEN_USBMAPFILE);
@@ -918,6 +951,17 @@ static void prtdepend(char *base_dir, int nflag)
 		fprintf(dep, "\n\n");
 	}
 
+	ptmod = modules;
+	fprintf(generic_string, "# module             id=string\n");
+	for (i = 0; i < n_modules; i++, ptmod++) {
+		int j;
+		char **pstr = ptmod->generic_string;
+		if (!ptmod->n_generic_string)
+			continue;
+		for (j = 0; j < ptmod->n_generic_string; j++, ++pstr)
+			fprintf(generic_string, "%-20s %s\n", shortname(ptmod->name), *pstr);
+	}
+
 	ptmod = modules;
 	fprintf(pcimap, "# pci module         vendor     device     subvendor  subdevice  class      class_mask driver_data\n");
 	for (i = 0; i < n_modules; i++, ptmod++) {
@@ -1000,6 +1044,8 @@ static void prtdepend(char *base_dir, int nflag)
 		}
 	}
 
+	if (generic_string != stdout)
+		fclose(generic_string);
 	if (pcimap != stdout)
 		fclose(pcimap);
 	if (isapnpmap != stdout)
