@@ -52,9 +52,11 @@
        Keith Owens <kaos@ocs.com.au> October 2000.
      Add insmod -S.
        Keith Owens <kaos@ocs.com.au> November 2000.
+     Add persistent data support.
+       Keith Owens <kaos@ocs.com.au> November 2000.
    */
 
-#ident "$Id: insmod.c 1.30.1.2 Tue, 14 Nov 2000 23:36:27 +1100 kaos $"
+#ident "$Id: insmod.c 1.32 Thu, 07 Dec 2000 22:40:49 +1100 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -687,6 +689,23 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 	}
 	free(absolute_filename);
 
+	/* record where the persistent data is going, same address as previous symbol */
+
+	if (f->persist) {
+		l = sizeof(symprefix)+		/* "__insmod_" */
+			lm_name+		/* module name */
+			2+			/* "_P" */
+			strlen(f->persist)+	/* data store */
+			1;			/* nul */
+		name = xmalloc(l);
+		snprintf(name, l, "%s%s_P%s",
+			 symprefix, m_name, f->persist);
+		sym = obj_add_symbol(f, name, -1, ELFW(ST_INFO) (STB_GLOBAL, STT_NOTYPE),
+				     sec->idx, sec->header.sh_addr, 0);
+		if (use_ksymtab)
+			add_ksymtab(f, sym);
+	}
+
 	/* tag the desired sections if size is non-zero */
 
 	for (i = 0; i < sizeof(section_names)/sizeof(section_names[0]); ++i) {
@@ -711,7 +730,7 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 	}
 }
 
-static int process_module_arguments(struct obj_file *f, int argc, char **argv)
+static int process_module_arguments(struct obj_file *f, int argc, char **argv, int required)
 {
 	for (; argc > 0; ++argv, --argc) {
 		struct obj_symbol *sym;
@@ -737,8 +756,15 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv)
 			memcpy(key + 5, *argv, n);
 			key[n + 5] = '\0';
 			if ((fmt = get_modinfo_value(f, key)) == NULL) {
-				error("invalid parameter %s", key);
-				return 0;
+				if (required) {
+					error("invalid parameter %s", key);
+					return 0;
+				}
+				else {
+					if (flag_verbose)
+						lprintf("ignoring %s", *argv);
+					continue;	/* silently ignore optional parameters */
+				}
 			}
 			key += 5;
 
@@ -1237,6 +1263,114 @@ static int old_init_module(const char *m_name, struct obj_file *f,
 /* end compat */
 /************************************************************************/
 
+/* Check that a module parameter has a reasonable definition */
+static int check_module_parameter(struct obj_file *f, char *key, char *value, int *persist_flag)
+{
+	struct obj_symbol *sym;
+	int min, max;
+	char *p = value;
+
+	sym = obj_find_symbol(f, key);
+	if (sym == NULL) {
+		error("symbol for parameter %s not found", key);
+		return(1);
+	}
+
+	if (isdigit(*p)) {
+		min = strtoul(p, &p, 10);
+		if (*p == '-')
+			max = strtoul(p + 1, &p, 10);
+		else
+			max = min;
+	} else
+		min = max = 1;
+
+	if (max < min) {
+		error("parameter %s has max < min!", key);
+		return(1);
+	}
+
+	switch (*p) {
+	case 'c':
+		if (!isdigit(p[1])) {
+			error("parameter %s has no size after 'c'!", key);
+			return(1);
+		}
+		while (isdigit(p[1]))
+			++p;	/* swallow c array size */
+		break;
+	case 'b':	/* drop through */
+	case 'h':	/* drop through */
+	case 'i':	/* drop through */
+	case 'l':	/* drop through */
+	case 's':
+		break;
+	case '\0':
+		error("parameter %s has no format character!", key);
+		return(1);
+	default:
+		error("parameter %s has unknown format character '%c'", key, *p);
+		return(1);
+	}
+	switch (*++p) {
+	case 'p':
+		if (*(p-1) == 's') {
+			error("parameter %s is invalid persistent string", key);
+			return(1);
+		}
+		*persist_flag = 1;
+		break;
+	case '\0':
+		break;
+	default:
+		error("parameter %s has unknown format modifier '%c'", key, *p);
+		return(1);
+	}
+	return(0);
+}
+
+/* Check that all module parameters have reasonable definitions */
+static void check_module_parameters(struct obj_file *f, int *persist_flag)
+{
+	struct obj_section *sec;
+	char *ptr, *value, *n, *endptr;
+	int namelen, err = 0;
+
+	sec = obj_find_section(f, ".modinfo");
+	if (sec == NULL) {
+		/* module does not support typed parameters */
+		return;
+	}
+
+	ptr = sec->contents;
+	endptr = ptr + sec->header.sh_size;
+	while (ptr < endptr && !err) {
+		value = strchr(ptr, '=');
+		n = strchr(ptr, '\0');
+		if (value) {
+			namelen = value - ptr;
+			if (namelen >= 5 && strncmp(ptr, "parm_", 5) == 0
+			    && !(namelen > 10 && strncmp(ptr, "parm_desc_", 10) == 0)) {
+				char *pname = xmalloc(namelen + 1);
+				strncpy(pname, ptr + 5, namelen - 5);
+				pname[namelen - 5] = '\0';
+				err = check_module_parameter(f, pname, value+1, persist_flag);
+				free(pname);
+			}
+		} else {
+			if (n - ptr >= 5 && strncmp(ptr, "parm_", 5) == 0) {
+				error("parameter %s found with no value", ptr);
+				err = 1;
+			}
+		}
+		ptr = n + 1;
+	}
+
+	if (err)
+		*persist_flag = 0;
+	return;
+}
+
 
 /* For common 3264 code, only compile the usage message once, in the 64 bit version */
 #if defined(COMMON_3264) && defined(ONLY_32)
@@ -1245,7 +1379,7 @@ extern void insmod_usage(void);		/* Use the copy in the 64 bit version */
 void insmod_usage(void)
 {
 	fputs("Usage:\n"
-	      "insmod [-fhkLmnpqrsSvVxXyY] [-o module_name] [-O blob_name] [-P prefix] module [ symbol=value ... ]\n"
+	      "insmod [-fhkLmnpqrsSvVxXyY] [-e persist_name] [-o module_name] [-O blob_name] [-P prefix] module [ symbol=value ... ]\n"
 	      "\n"
 	      "  module                Name of a loadable kernel module ('.o' can be omitted)\n"
 	      "  -f, --force           Force loading under wrong kernel version\n"
@@ -1265,6 +1399,8 @@ void insmod_usage(void)
 	      "  -X, --export          Do export externs (default)\n"
 	      "  -y, --noksymoops      Do not add ksymoops symbols\n"
 	      "  -Y, --ksymoops        Do add ksymoops symbols (default)\n"
+	      "  -e persist_name\n"
+	      "      --persist=persist_name Filename to hold any persistent data from the module\n"
 	      "  -o NAME, --name=NAME  Set internal module name to NAME\n"
 	      "  -O NAME, --blob=NAME  Save the object as a binary blob in NAME\n"
 	      "  -P PREFIX\n"
@@ -1306,6 +1442,8 @@ int INSMOD_MAIN(int argc, char **argv)
 		{"export", 0, 0, 'X'},
 		{"noksymoops", 0, 0, 'y'},
 		{"ksymoops", 0, 0, 'Y'},
+
+		{"persist", 1, 0, 'e'},
 		{"name", 1, 0, 'o'},
 		{"blob", 1, 0, 'O'},
 		{"prefix", 1, 0, 'P'},
@@ -1319,6 +1457,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	int m_crcs;
 	char m_strversion[STRVERSIONLEN];
 	char *filename;
+	char *persist_name = NULL;	/* filename to hold any persistent data */
 	int fp;
 	struct obj_file *f;
 	struct obj_section *kallsyms = NULL, *archdata = NULL;
@@ -1328,6 +1467,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	int quiet = 0;
 	int exit_status = 1;
 	int force_kallsyms = 0;
+	int persist_parms = 0;	/* does module have persistent parms? */
 	int i;
 
 	error_file = "insmod";
@@ -1336,7 +1476,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	errors = optind = 0;
 
 	/* Process the command line.  */
-	while ((o = getopt_long(argc, argv, "fhkLmnpqrsSvVxXyYo:O:P:",
+	while ((o = getopt_long(argc, argv, "fhkLmnpqrsSvVxXyYe:o:O:P:R:",
 				&long_opts[0], NULL)) != EOF)
 		switch (o) {
 		case 'f':	/* force loading */
@@ -1391,6 +1531,10 @@ int INSMOD_MAIN(int argc, char **argv)
 			flag_ksymoops = 1;
 			break;
 
+		case 'e':	/* persistent data filename */
+			free(persist_name);
+			persist_name = xstrdup(optarg);
+			break;
 		case 'o':	/* name the output module */
 			m_name = optarg;
 			break;
@@ -1415,6 +1559,14 @@ int INSMOD_MAIN(int argc, char **argv)
 		error("Failed handle configuration");
 	}
 
+	if (persist_name && !*persist_name &&
+	    (!persistdir || !*persistdir)) {
+		free(persist_name);
+		persist_name = NULL;
+		if (flag_verbose)
+			lprintf("insmod: -e \"\" ignored, no persistdir");
+	}
+
 	if (m_name == NULL) {
 		size_t len;
 		char *p;
@@ -1453,7 +1605,6 @@ int INSMOD_MAIN(int argc, char **argv)
 
 	/* And open it.  */
 	if ((fp = gzf_open(filename, O_RDONLY)) == -1) {
-		error_file = filename;
 		error("%s: %m", filename);
 		return 1;
 	}
@@ -1534,12 +1685,128 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 	obj_allocate_commons(f);	/* DEPMOD */
 
+	check_module_parameters(f, &persist_parms);
+
 	if (optind < argc) {
-		if (!process_module_arguments(f, argc - optind, argv + optind))
+		if (!process_module_arguments(f, argc - optind, argv + optind, 1))
 			goto out;
 	}
 	arch_create_got(f);	/* DEPMOD */
 	hide_special_symbols(f);
+
+	if (persist_parms && persist_name && *persist_name) {
+		f->persist = persist_name;
+		persist_name = NULL;
+	}
+
+	if (persist_parms &&
+	    persist_name && !*persist_name) {
+		/* -e "".  This is ugly.  Take the filename, compare it against
+		 * each of the module paths until we find a match on the start
+		 * of the filename, assume the rest is the relative path.  Have
+		 * to do it this way because modprobe uses absolute filenames
+		 * for module names in modules.dep and the format of modules.dep
+		 * does not allow for any backwards compatible changes, so there
+		 * is nowhere to store the relative filename.  The only way this
+		 * should fail to calculate a relative path is "insmod ./xxx", for
+		 * that case the user has to specify -e filename.
+		 */
+		int j, l = strlen(filename);
+		char *relative = NULL;
+		char *p;
+		for (i = 0; i < nmodpath; ++i) {
+			p = modpath[i].path;
+			j = strlen(p);
+			while (j && p[j] == '/')
+				--j;
+			if (j < l && strncmp(filename, p, j) == 0 && filename[j] == '/') {
+				while (filename[j] == '/')
+					++j;
+				relative = xstrdup(filename+j);
+				break;
+			}
+		}
+		if (relative) {
+			i = strlen(relative);
+			if (i > 3 && strcmp(relative+i-3, ".gz") == 0)
+				relative[i -= 3] = '\0';
+			if (i > 2 && strcmp(relative+i-2, ".o") == 0)
+				relative[i -= 2] = '\0';
+			else if (i > 4 && strcmp(relative+i-4, ".mod") == 0)
+				relative[i -= 4] = '\0';
+			f->persist = xmalloc(strlen(persistdir) + 1 + i + 1);
+			strcpy(f->persist, persistdir);	/* safe, xmalloc */
+			strcat(f->persist, "/");	/* safe, xmalloc */
+			strcat(f->persist, relative);	/* safe, xmalloc */
+			free(relative);
+		}
+		else
+			error("Cannot calculate persistent filename");
+	}
+
+	if (f->persist && *(f->persist) != '/') {
+		error("Persistent filenames must be absolute, ignoring '%s'",
+			f->persist);
+		free(f->persist);
+		f->persist = NULL;
+	}
+
+	if (f->persist && !flag_ksymoops) {
+		error("has persistent data but ksymoops symbols are not available");
+		free(f->persist);
+		f->persist = NULL;
+	}
+
+	if (f->persist && !k_new_syscalls) {
+		error("has persistent data but the kernel is too old to support it");
+		free(f->persist);
+		f->persist = NULL;
+	}
+
+	if (persist_parms && flag_verbose) {
+		if (f->persist)
+			lprintf("Persist filename '%s'", f->persist);
+		else
+			lprintf("No persistent filename available");
+	}
+
+	if (f->persist) {
+		FILE *fp = fopen(f->persist, "r");
+		if (!fp) {
+			if (flag_verbose)
+				lprintf("Cannot open persist file '%s' %m", f->persist);
+		}
+		else {
+			int pargc = 0;
+			char *pargv[1000];	/* hard coded but big enough */
+			char line[3000];	/* hard coded but big enough */
+			char *p;
+			while (fgets(line, sizeof(line), fp)) {
+				p = strchr(line, '\n');
+				if (!p) {
+					error("Persistent data line is too long\n%s", line);
+					break;
+				}
+				*p = '\0';
+				p = line;
+				while (isspace(*p))
+					++p;
+				if (!*p || *p == '#')
+					continue;
+				if (pargc == sizeof(pargv)/sizeof(pargv[0])) {
+					error("More than %d persistent parameters", pargc);
+					break;
+				}
+				pargv[pargc++] = xstrdup(p);
+			}
+			fclose(fp);
+			if (!process_module_arguments(f, pargc, pargv, 0))
+				goto out;
+			while (pargc--)
+				free(pargv[pargc]);
+		}
+	}
+
 	if (flag_ksymoops)
 		add_ksymoops_symbols(f, filename, m_name);
 
@@ -1597,6 +1864,29 @@ int INSMOD_MAIN(int argc, char **argv)
 		}
 	}
 
+	/* module is already built, complete with ksymoops symbols for the
+	 * persistent filename.  If the kernel does not support persistent data
+	 * then give an error but continue.  It is too difficult to clean up at
+	 * this stage and this error will only occur on backported modules.
+	 * rmmod will also get an error so warn the user now.
+	 */
+	if (f->persist && !noload) {
+		struct {
+			struct module m;
+			int data;
+		} test_read;
+		memset(&test_read, 0, sizeof(test_read));
+		test_read.m.size_of_struct = -sizeof(test_read.m);      /* -ve size => read, not write */
+		test_read.m.read_start = m_addr + sizeof(struct module);
+		test_read.m.read_end = test_read.m.read_start + sizeof(test_read.data);
+		if (sys_init_module(m_name, (struct module *) &test_read)) {
+			int old_errors = errors;
+			error("has persistent data but the kernel is too old to support it."
+				"  Expect errors during rmmod as well");
+			errors = old_errors;
+		}
+	}
+
 	if (!obj_relocate(f, m_addr)) {	/* DEPMOD */
 		if (!noload)
 			delete_module(m_name);
