@@ -29,9 +29,11 @@
  *   Keith Owens <kaos@ocs.com.au> December 1999.
  * Add arch64().
  *   Keith Owens <kaos@ocs.com.au> December 1999.
+ * Print " persistent" for persistent parameters.
+ *   Keith Owens <kaos@ocs.com.au> November 2000.
  */
 
-#ident "$Id: modinfo.c 1.11.1.2 Tue, 14 Nov 2000 23:36:27 +1100 kaos $"
+#ident "$Id: modinfo.c 1.13 Thu, 07 Dec 2000 22:40:49 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -290,6 +292,18 @@ static void show_parameter(struct obj_file *f, char *key, char *value,
 		printf("unknown format character '%c'", *p);
 		return;
 	}
+	switch (*++p) {
+	case 'p':
+		printf(" persistent");
+		if (*(p-1) == 's')
+			printf(" (invalid)");
+		break;
+	case '\0':
+		break;
+	default:
+		printf(" unknown format modifier '%c'", *p);
+		break;
+	}
 
 	if (min > 1 || max > 1)
 		printf(" array (min = %d, max = %d)", min, max);
