@@ -98,6 +98,10 @@ static int n_probeall_list;
 OPT_LIST *aliases;
 static int n_aliases;
 
+char *persistdir = "/var/lib/modules/persist";
+
+const char symprefix[] = SYMPREFIX;
+
 char *insmod_opt = NULL;
 char *config_file = NULL;	/* Which file was actually used */
 time_t config_mtime;
@@ -107,6 +111,7 @@ int quick = 0;			/* Option -A */
 
 /* The initialization order must match the gen_file_enum order in config.h */
 struct gen_files gen_file[] = {
+	{"generic_string", NULL, 0},
 	{"pcimap", NULL, 0},
 	{"isapnpmap", NULL, 0},
 	{"usbmap", NULL, 0},
@@ -402,16 +407,16 @@ static int build_list(char **in, OPT_LIST **out, char *version, int opts)
 /* Environment variables can override defaults, testing only */
 static void gen_file_env(struct gen_files *gf)
 {
-        char *e = xmalloc(strlen(gf->base)+5), *p1 = gf->base, *p2 = e;
-	if (safemode)
-		return;
-	while ((*p2++ = toupper(*p1++))) ;
-	strcpy(p2-1, "PATH");	/* safe, xmalloc */
-	if ((p2 = getenv(e)) != NULL) {
-		free(gf->name);
-		gf->name = xstrdup(p2);
+	if (!safemode) {
+		char *e = xmalloc(strlen(gf->base)+5), *p1 = gf->base, *p2 = e;
+		while ((*p2++ = toupper(*p1++))) ;
+		strcpy(p2-1, "PATH");	/* safe, xmalloc */
+		if ((p2 = getenv(e)) != NULL) {
+			free(gf->name);
+			gf->name = xstrdup(p2);
+		}
+		free(e);
 	}
-	free(e);
 }
 
 /* Read a config option for a generated filename */
@@ -584,6 +589,9 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	 * include FILE_TO_INCLUDE
 	 *	This does what you expect. No limitation on include levels.
 	 *
+	 * persistdir=persist_directory
+	 *	Name the directory to save persistent data from modules.
+	 *
 	 * In the following WORD is a sequence if non-white characters.
 	 * If ' " or ` is found in the string, all characters up to the
 	 * matching ' " or ` will also be included, even whitespace.
@@ -1304,6 +1312,15 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			one_err = 0;
 		}
 
+		/*
+		 * persistdir
+		 */
+		else if (assgn && strcmp(parm, "persistdir") == 0) {
+			meta_expand(arg, &g, NULL, version, ME_ALL);
+			persistdir = xstrdup(g.pathc ? g.pathv[0] : arg);
+			one_err = 0;
+		}
+
 		/* Names for generated files in config file */
 		for (i = 0; one_err && i < gen_file_count; ++i)
 			one_err = gen_file_conf(gen_file+i, assgn, parm, arg);
